#define com_maroontress_libcgi_db_Tree_IMPORT
#define com_maroontress_libcgi_db_Box_IMPORT

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <com/maroontress/libcgi/db/Tree.h>
#include <com/maroontress/libcgi/db/Box.h>
#include "BuildCharset.h"
#include "Serial.h"
#include "Project.h"
#include "Platform.h"

struct BuildCharset {
    struct Project *project;
    struct Serial *serial;
    char *charset;
    int platformCode;
};

static int
loadBuildCharsetWithTree(struct BuildCharset *cs, struct Tree *builds)
{
    struct Box *box;
    int k, platforms;
    char *charset;

    if ((box = Tree_getBox(builds, Serial_getKey(cs->serial))) == NULL
        || Box_getInt(box, "platforms", &platforms) < 0
	|| cs->platformCode >= platforms) {
        return -1;
    }
    k = cs->platformCode;

#ifdef WITH_PLATFORM_CHARSET
    if (Platform_getStringWithBox(box, k, "charset", &charset) < 0
	|| (cs->charset = strdup(charset)) == NULL) {
	return -1;
    }
#else
    if (1) {
	char defaultCharset[] = "UTF-8";
	if (Platform_getStringWithBox(box, k, "charset", &charset) < 0) {
	    charset = defaultCharset;
	}
	if ((cs->charset = strdup(charset)) == NULL) {
	    return -1;
	}
    }
#endif
    return 0;
}

static int
loadBuildCharset(struct BuildCharset *cs)
{
    struct Tree *builds;
    int s;

    if ((builds = Tree_create(Project_getDBPath(cs->project),
                              Tree_MODE_SHARED)) == NULL) {
        return -1;
    }
    s = loadBuildCharsetWithTree(cs, builds);
    Tree_free(builds);
    return s;
}

struct BuildCharset *
BuildCharset_create(const char *projectID, const char *serial,
		    const char *platformCode)
{
    struct BuildCharset *cs;

    if ((cs = (struct BuildCharset *)malloc(sizeof(*cs))) == NULL) {
	return NULL;
    }
    cs->project = NULL;
    cs->serial = NULL;
    cs->charset = NULL;

    if ((cs->project = Project_create(projectID)) == NULL
	|| (cs->serial = Serial_create(atoi(serial))) == NULL
	|| (cs->platformCode = atoi(platformCode)) < 0
	|| loadBuildCharset(cs) < 0) {
	BuildCharset_free(cs);
        return NULL;
    }
    return cs;
}

void
BuildCharset_free(struct BuildCharset *cs)
{
    if (cs == NULL)
        return;
    Project_free(cs->project);
    Serial_free(cs->serial);
    free(cs->charset);
    free(cs);
}

char *
BuildCharset_getCharset(struct BuildCharset *cs)
{
    return cs->charset;
}
